﻿{
	/*
		Null Swapper 2.0  ©2012-2020
		Developer: David Torno
		Description: A simple script to swap Null layers for Light layers.
	*/
	
	function nullSwapper(thisObj){
		var getAEVerData = aeVer();
		var userAEVer = getAEVerData.major + "." + getAEVerData.minor;
		if(userAEVer >= 10.5){
		
			var nsObj = new Object();
			nsObj.scriptDev = "David Torno";
			nsObj.scriptVersion = "2.0";
			nsObj.scriptName = "Null Swapper";
			nsObj.statusSwapping = "Swapping";
			nsObj.statusTransfer = "Tranferring";
			nsObj.statusComplete = "Swap complete";
		
			function nullSwapper_buildUI(thisObj){
				var pal = (thisObj instanceof Panel) ? thisObj : new Window("palette", nsObj.scriptName, undefined, {resizeable:true});
				if (pal != null){
					var res ="group {orientation:'column', alignment:['fill','fill'], alignChildren:['fill','top'],\
							mainUIGrp: Group{orientation:'column', alignment:['fill','top'], alignChildren:['fill','top'],\
								butGroup: Group{orientation:'row', alignment:['fill','top'], alignChildren:['fill','top'],\
									swapBut: Button{text:'Swap', alignment:['left','top'] },\
									lightChoice: DropDownList{properties:{items:['Parallel', 'Spot', 'Point']}, alignment:['left','top'] },\
								},\
							},\
					}";
					
					pal.grp = pal.add(res);
					
					pal.layout.layout(true);
					pal.grp.minimumSize = pal.grp.size;
					pal.layout.resize();
					pal.onResizing = pal.onResize = function() {this.layout.resize();}
					
					///UI VARIABLES
					var h = pal.grp;
					var butGroup = h.mainUIGrp.butGroup;
					var swapBut = butGroup.swapBut;
					var lightChoice = butGroup.lightChoice;
					
					///DEFAULTS
					lightChoice.selection = 2;
				
					///ONCLICKS
					swapBut.onClick = function(){swapIt(lightChoice)};
					
					///FUNCTIONS
					function swapIt(lightChoice){
						var useLightType, lc;
						lc = lightChoice.selection.text;

						switch(lc){	//Determine chosen light type object to use
							case "Parallel":
								useLightType = LightType.PARALLEL;
								break;
							case "Spot":
								useLightType = LightType.SPOT;
								break;
							case "Point":
								useLightType = LightType.POINT;
								break;
							default:
								useLightType = LightType.POINT;
								break;
						}

						var curComp = app.project.activeItem;
						if(curComp instanceof CompItem){
							var myLayers = curComp.selectedLayers;
							if(myLayers.length == 0 || myLayers == null){	//Nothing selected warn user
								alert("Please select all of the Nulls you want swapped.");
							}else{	//Begin swapping process
								writeLn("Swapping...");
								var nullPosition, nullName, xyz, newLight, oldPositionProp, newPositionProp, totalKeys;
								var twoDNulls = new Array();
								var curKey = 0;
								
								app.beginUndoGroup("Null Swapper");	//START UNDO GROUP
									for (var i = 0; i < myLayers.length; i++){
										var myLayer = myLayers[i];
										var selLayers = myLayer;

										if(selLayers.nullLayer == true && selLayers.threeDLayer == true){	//Handle 3D Nulls
											writeLn(nsObj.statusSwapping + " - " + myLayer.name);
											nullPosition = selLayers.property("ADBE Transform Group").property("ADBE Position").value;
											xyz = nullPosition.toString().split(",");
											nullName = selLayers.name;
											newLight = curComp.layers.addLight(nullName, [xyz[0], xyz[1]]);
											newLight.moveBefore(selLayers);
											newLight.lightType = useLightType;
											newLight.enabled = true;
											newLight.property("ADBE Transform Group").property("ADBE Position").setValue([xyz[0], xyz[1], xyz[2]]);
											if(selLayers.property("Transform").property("Position").isTimeVarying){
												oldPositionProp = selLayers.property("Transform").property("Position");
												newPositionProp = newLight.property("Transform").property("Position");
												writeLn(nsObj.statusTransfer);
												totalKeys = oldPositionProp.numKeys.toString();
												for(var o=oldPositionProp.numKeys; o>0; o--){
													writeLn(nsObj.statusTransfer + o.toString()+" of "+totalKeys);
													copyKeyframeToNewController(oldPositionProp, oldPositionProp.nearestKeyIndex(curKey), newPositionProp, oldPositionProp.nearestKeyIndex(curKey));
												}
											}

											selLayers.remove();

										}else if(selLayers.nullLayer == true && selLayers.threeDLayer == false){	//Handle 2D Nulls
											writeLn(nsObj.statusSwapping + " - " + myLayer.name);
											twoDNulls.push(selLayers.index);
											nullPosition = selLayers.property("ADBE Transform Group").property("ADBE Position").value;
											xyz = nullPosition.toString().split(",");
											nullName = selLayers.name;
											newLight = curComp.layers.addLight(nullName, [xyz[0], xyz[1]]);
											newLight.moveBefore(selLayers);
											newLight.lightType = useLightType;
											newLight.property("ADBE Transform Group").property("ADBE Position").setValue([xyz[0], xyz[1], 0]);
											newLight.enabled = true;

											if(selLayers.property("Transform").property("Position").isTimeVarying){
												oldPositionProp = selLayers.property("Transform").property("Position");
												newPositionProp = newLight.property("Transform").property("Position");
												totalKeys = oldPositionProp.numKeys.toString();
												for(var n=oldPositionProp.numKeys; n>0; n--){
													writeLn(nsObj.statusTransfer + n.toString()+" of "+totalKeys);
													copyKeyframeToNewController(oldPositionProp, oldPositionProp.nearestKeyIndex(curKey), newPositionProp, oldPositionProp.nearestKeyIndex(curKey));
												}
											}

											selLayers.remove();
										
										}
									}
								app.endUndoGroup();	//END UNDO GROUP
								
								if(twoDNulls.length > 0){	//Tell user if 2D data was detected
									writeLn(nsObj.statusComplete);
									alert("Some of the selected Nulls were 2D and not 3D, so only X and Y data was transferred. Z data was left at the default value of 0.");
								}else{
									writeLn(nsObj.statusComplete);
								}
							}
						}else{
							alert("There is no open composition currently available.");
						}
					}

					function copyKeyframeToNewController(oldProp, keyToCopy, newProp, keyToRemove){
						// Remember the key's settings before creating the new setting, just in case creating the new key affects keyToCopy's settings
						var inInterp = oldProp.keyInInterpolationType(keyToCopy);
						var outInterp = oldProp.keyOutInterpolationType(keyToCopy);
						
						if ((inInterp == KeyframeInterpolationType.BEZIER) && (outInterp == KeyframeInterpolationType.BEZIER)){
							var tempAutoBezier = oldProp.keyTemporalAutoBezier(keyToCopy);
							var tempContBezier = oldProp.keyTemporalContinuous(keyToCopy);
						}
						if (outInterp != KeyframeInterpolationType.HOLD){
							var inTempEase = oldProp.keyInTemporalEase(keyToCopy);
							var outTempEase = oldProp.keyOutTemporalEase(keyToCopy);
						}
						if ((oldProp.propertyValueType == PropertyValueType.TwoD_SPATIAL) || (oldProp.propertyValueType == PropertyValueType.ThreeD_SPATIAL)){
							var spatAutoBezier = oldProp.keySpatialAutoBezier(keyToCopy);
							var spatContBezier = oldProp.keySpatialContinuous(keyToCopy);
							var inSpatTangent = oldProp.keyInSpatialTangent(keyToCopy);
							var outSpatTangent = oldProp.keyOutSpatialTangent(keyToCopy);
							var roving = oldProp.keyRoving(keyToCopy);
						}
						
						// Create the new keyframe...
						var newTime = oldProp.keyTime(keyToCopy);
						var newKeyIndex = newProp.addKey(newTime);
						// ...on new layer property.
						newProp.setValueAtKey(newKeyIndex, oldProp.keyValue(keyToCopy));
						
						if(outInterp != KeyframeInterpolationType.HOLD){
							newProp.setTemporalEaseAtKey(newKeyIndex, inTempEase, outTempEase);
						}
						
						// Copy over the keyframe settings
						newProp.setInterpolationTypeAtKey(newKeyIndex, inInterp, outInterp);
						
						if ((inInterp == KeyframeInterpolationType.BEZIER) && (outInterp == KeyframeInterpolationType.BEZIER) && tempContBezier){
							newProp.setTemporalContinuousAtKey(newKeyIndex, tempContBezier);

							newProp.setTemporalAutoBezierAtKey(newKeyIndex, tempAutoBezier);		// Implies Continuous, so do after it
						}
						
						if ((oldProp.propertyValueType == PropertyValueType.TwoD_SPATIAL) || (oldProp.propertyValueType == PropertyValueType.ThreeD_SPATIAL)){

							newProp.setSpatialContinuousAtKey(newKeyIndex, spatContBezier);
							newProp.setSpatialAutoBezierAtKey(newKeyIndex, spatAutoBezier);		// Implies Continuous, so do after it
							newProp.setSpatialTangentsAtKey(newKeyIndex, inSpatTangent, outSpatTangent);
							newProp.setRovingAtKey(newKeyIndex, roving);
						}
						
						// Remove the old keyframe
						oldProp.removeKey(keyToRemove);
					}

				}
				
				return pal;
			}//End buildUI
			
			
			var nsPal = nullSwapper_buildUI(thisObj);
			if(nsPal != null){
				if(nsPal instanceof Window){
					nsPal.center();
					nsPal.show();
				}
			}
		}else{
			alert("This script requires After Effects CS 5.5 or later.");
		}
		function aeVer(){	//Custom function to get After Effects version number
				var ae, aeFullVer, aeFullVerSplit, verSec, full, major, minor, subMinor, build;
				var aeFullVer = app.version;
				ae = aeFullVer.split("x");
				verSec = ae[0].split(".");
				major = verSec[0];
				minor = subMinor = null;
				(ae.length > 1) ? build = ae[1] : build = null;
				switch(verSec.length){
					case 1:/*Major only with forced 0 for Minor*/
						full = major + ".0";
						break;
					case 2:/*Major & Minor only*/
						minor = verSec[1];
						full = (major + "." + minor);
						break;
					case 3:/*Major, Minor, & Maintainance*/
						minor = verSec[1];
						subMinor = verSec[2];
						full = (major + "." + minor + "." + subMinor);
						break;
				}
				return {
					'raw': aeFullVer,
					'major': major,
					'minor': minor,
					'subMinor': subMinor,
					'build': build,
					'full': full
				};
			}
	}
	nullSwapper(this);
}